require 'fileutils'
require 'yaml'
require 'pathname'
require 'cocoapods/downloader'
require 'cocoapods/config'

require_relative('podfile_switch')

# noinspection RubyInterpreter
Pod::HooksManager.register('cocoapods-cafswitcher', :pre_install) do |installer_content|
  ## 处理所有命令的优先级关系
  #
  # 对于全局变量
  # all_framework! > all_archive! > all_source!
  # 如果不指定，默认是framwork的形式
  #
  #
  # 对与有针对特定的pod库的局部变量
  # isFramework > isArchive > isSource
  #
  #
  # 如果全局变量和局部变量同时出现
  # 全部变量统领全局
  # 布局变量针对特殊pod库
  #
  # 如果对两个相同的库。如：
  # LJCache/Foundation 和 LJCache/UIKit 算两个库，可以分别进行控制
  # 如果出现：
  #   all_archive!
  #   pod 'LJCache/Foundation', isFramework => true
  #   pod 'LJCache/UIKit'
  #
  # 则 LJCache/Foundation 安装为framework
  #    LJCache/UIKit 安装为archive
  #
  #
  allNames = Pod::Podfile.allPodName.uniq
  gitPathList = Pod::Podfile.gitPathList.uniq
  currentSourceList = Pod::Podfile.sourceList.uniq
  currentFrameworkList = Pod::Podfile.frameworkList.uniq
  currentUnspecifiedList = Pod::Podfile.unspecifiedList.uniq

  is_Source = false
  if ENV['IS_SOURCE']
    is_Source = true
    ENV.delete('IS_SOURCE')
    allNames.each do |name|
      setEnv(name, '_SOURCE')
      # Pod::Podfile.tmpSourceList << name
    end
  end

  isDebug = true
  if ENV['IS_RELEASE']
    isDebug = false
  end

  # 对debug和release切换，release级别高
  # for debugName in Pod::Podfile.debugPackageList.uniq
  #   cleanDebugENV(debugName)
  #   setEnv(debugName, "_DEBUG")
  # end
  #
  # for releaseName in Pod::Podfile.releasePackageList.uniq
  #   cleanDebugENV(releaseName)
  #   setEnv(releaseName, "_RELEASE")
  # end

  # 二进制切换
  currentSourceList.each do |sourceName|
    cleanENV(sourceName)
    setEnv(sourceName, '_SOURCE')
  end

  currentFrameworkList.each do |frameworkName|
    cleanENV(frameworkName)
    setEnv(frameworkName, '_FRAMEWORK')
  end

  currentUnspecifiedList.each do |unspecifiedName|
    next unless !is_Source
    cleanENV(unspecifiedName)
    if gitPathList.include?(unspecifiedName)
      setEnv(unspecifiedName, '_SOURCE')
    else
      setEnv(unspecifiedName, '_FRAMEWORK')
    end
  end


  sourceList = []
  frameworkList = []
  debugPackageList = Array.new
  releasePackageList = Array.new

  allNames.each do |name|
    # 对于区分subspec的库，subspec之间只能同时是debug或者release
    envName = name.gsub(/[\/]/, '_')
    if ENV["IS_RELEASE"] == '1'
      releasePackageList << name
    else
      debugPackageList << name
    end

    if ENV["#{envName}_SOURCE"] == '1'
      sourceList << name
    else
      frameworkList << name
    end
  end
  Pod::UI.puts "\n"
  Pod::UI.puts "is Source ：\n   #{sourceList}"
  Pod::UI.puts "is Framework ：\n   #{frameworkList}"

  Pod::UI.puts "debugPackage tag ：\n   #{debugPackageList}"
  Pod::UI.puts "releasePackage tag：\n   #{releasePackageList}"

  # 保存最终设置
  Pod::Podfile.sourceList.clear()
  Pod::Podfile.frameworkList.clear()
  Pod::Podfile.debugPackageList.clear()
  Pod::Podfile.releasePackageList.clear()
  Pod::Podfile.unspecifiedList.clear()

  Pod::Podfile.sourceList.concat(sourceList)
  Pod::Podfile.frameworkList.concat(frameworkList)
  Pod::Podfile.debugPackageList.concat(debugPackageList)
  Pod::Podfile.releasePackageList.concat(releasePackageList)

  # 如果某一个Pod和上一次比发生变化，需要清除缓存
  # 1. 清除pod缓存
  # pod cache clean LJCache --all
  # 2. 清除壳工程的pod文件夹下的缓存（如果指向的是path，即本地，不做处理）
  # rm -r -f Pod/LJCache
  # 3. 保存本次配置


  # 现在的状态
  caf_hash = Hash['sourceList' => sourceList, 'frameworkList' => frameworkList,
                  'debugPackageList' => debugPackageList, 'releasePackageList' => releasePackageList ]


  config = Pod::Config.instance

  require_relative('cafswitcher_config')

  configPath = config.cafConfigPath
  FileUtils.mkdir_p(configPath) unless configPath.exist?
  configFile = config.cafConfigFile
  # 旧的状态
  cafConfig = Hash['sourceList' => [], 'frameworkList' => [],
                   'debugPackageList' => [], 'releasePackageList' => []]
  if configFile.exist?
    cafConfigLockContent = YAML.load(File.open(configFile))
    if cafConfigLockContent
      cafConfig = cafConfigLockContent
    end
  else
    FileUtils.touch configFile
  end
  # 比较不同
  changedSubPods = []
  caf_hash.each do |key, value|
    if key == 'debugPackageList'
      cafDebugList = []
      if cafConfig.include?('debugPackageList')
        cafDebugList = cafConfig['debugPackageList']
      end
      changedSubPods |= (value | cafDebugList) - (value & cafDebugList)
    elsif key == 'releasePackageList'
      cafReleaseeList = []
      if cafConfig.include?('releasePackageList')
        cafReleaseeList = cafConfig['releasePackageList']
      end
      changedSubPods |= (value | cafReleaseeList) - (value & cafReleaseeList)
    elsif key == 'sourceList'
      cafSourceList = []
      if cafConfig.include?('sourceList')
        cafSourceList = cafConfig['sourceList']
      end
      changedSubPods |= (value | cafSourceList) - (value & cafSourceList)
    elsif key == 'frameworkList'
      cafFrameworkList = []
      if cafConfig.include?('frameworkList')
        cafFrameworkList = cafConfig['frameworkList']
      end
      changedSubPods |= (value | cafFrameworkList) - (value & cafFrameworkList)
    end
  end

  Pod::UI.puts "\nchanged sub Pod is:\n   " + changedSubPods.to_s unless changedSubPods.empty?
  # changedPod 里是根据subspec来存的，需要进行合并，找到pod库而不是含有subspec的库

  changedPods = []
  changedSubPods.each do |subPod|
    podName  = subPod.split('/')[0]
    changedPods << podName
  end

  # gitPathList.each do |gitPathSubPod|
  #   changedSubPods << gitPathSubPod
  # end
  changedSubPods.uniq!

  changedPods.uniq!


  Pod::UI.puts "\nchanged Pod is:\n   " + changedPods.to_s + "\n" unless changedPods.empty?



  cleanPodsDir(changedSubPods)
  cleanCocoapodsCache(gitPathList)

  # 把新的配置写入file
  File.open(configFile, 'wb') {|f| YAML.dump(caf_hash, f) } unless changedSubPods.empty?
end

def cleanCocoapodsCache(gitPathList)
  cachePath = Pod::Config.instance.cache_root + 'Pods'
  @cache = Pod::Downloader::Cache.new(cachePath)
  gitPathList.each do |gitPathSubPod|
    gitPathPod = gitPathSubPod.split('/')[0]
    cache_descriptors = @cache.cache_descriptors_per_pod[gitPathPod]
    # 清空缓存，因为缓存里有podspec.json
    if cache_descriptors
      # TODO 检查changedSubPod
      cache_descriptors.each do |desc|
        specs_dir = cachePath + 'Specs'
        release_specs_dir = specs_dir + 'Release'
        if desc[:spec_file].to_s.include?(release_specs_dir.to_s)
          next
        end
        Pod::UI.message("Removing spec #{desc[:spec_file]} (v#{desc[:version]})") do
          FileUtils.rm_rf([desc[:spec_file]])
        end
        # Pod::UI.message("Removing cache #{desc[:slug]}") do
        #   FileUtils.rm_rf([desc[:slug]])
        # end
      end
    end
  end

end

def cleanPodsDir(changedSubPods)
  config = Pod::Config.instance
  # 根据不同删除cache
  podRoot = config.project_pods_root
  changedSubPods.each do |changedSubPod|
    changedPod = changedSubPod.split('/')[0]
    # 清除本地的localFile 的JSON
    localspecPath = podRoot + "/Local Podspecs/" + "#{changedPod}.podspec.json"
    if File.exist? localspecPath.to_s
      Pod::UI.message "Removing local Podspecs.json #{localspecPath}"
      FileUtils.rm_r(localspecPath)
    end
    # next unless changedSubPod =~ /^(?i)LJ/ || changedSubPod =~ /^(?i)Lianjia/ || changedSubPod.include?('/')
    # 清除目录 pods 文件夹下的缓存
    changedPodPath = podRoot + changedPod
    if File.exist? changedPodPath.to_s
      Pod::UI.message "Removing Folder #{changedPodPath}"
      FileUtils.rm_r(changedPodPath)
    end
  end
end


def setEnv(name, suffix)
  envName = name.gsub(/[\/]/, '_')
  ENV["#{envName}#{suffix}"] = '1'
  podName = name.split('/')[0]
  ENV["#{podName}#{suffix}"] = '1'
end



def cleanENV(name)
  envName = name.gsub(/[\/]/, '_')
  ENV.delete("#{envName}_SOURCE")
  ENV.delete("#{envName}_FRAMEWORK")
  ENV.delete('IS_SOURCE')
end

def cleanDebugENV(name)
  ENV.delete("#{name}_DEBUG")
  ENV.delete("#{name}_RELEASE")
end